!
! Crank-Nicholson Scheme. Linear one-dimensional advection equation.
! @author ttnguyen 
! Nguyen Tong Tam, group MB-457
! Version 23.03.2010
! 

program CrankNicholsonScheme 

  implicit none
  integer :: s
  integer :: q
  integer :: i
  integer :: j
  integer :: k
  integer :: g
  integer :: h
  real :: dr
  real :: dt
  real :: r
  real :: t
  real :: m
  real :: muy
  real :: C
  real :: L
  real, dimension (0:144, 0:100) :: Ftr
  real, dimension (0:98, 0:98) :: Tdmatrix
  real, dimension (0:98) :: f
  real, dimension (0:98) :: RootOfEquation
  real, dimension (0:98) :: alpha
  real, dimension (0:98) :: beta
  open (1, file = "CrankNicholsonScheme.dat")
  
  !Задание констант
  s = 144
  q = 100
  dr = 50
  dt = 10
  C = 20
  L = 1000

  !Волновое число и 
  m = 2 * 3.1415 / L;
  muy = C * dt * 60 / (dr * 1000);

  !Задание начального условия
  do j = 0, q
    Ftr(0, j) = 50 * cos(m * (j + 1) * dr) + 100
  end do
  !Задание граничных условий
  do i = 1, s
    Ftr(i, 0) = Ftr(0, 0)
    Ftr(i, q) = Ftr(0, q) 
  end do

  !Вычисление значений функции в узлах сетки на каждом шаге по времени      
  
  do i = 1, s

  !Задание элементов трехдиагональной матрицы
    !Первая строка 
    Tdmatrix(0, 0) = 1
    Tdmatrix(0, 1) = muy / 4
    f(0) = Ftr(i - 1, 1) - (muy / 4) * (-Ftr(i, 0) + Ftr(i - 1, 2) - Ftr(i - 1, 0))            
    !Подследняя строка
    Tdmatrix(q - 2, q - 3) = - muy / 4
    Tdmatrix(q - 2, q - 2) = 1
    f(q - 2) = Ftr(i - 1, q - 1) - (muy / 4) * (Ftr(i, q) + Ftr(i - 1, q) - Ftr(i - 1, q - 2))          
    !j-ая строка
    do j = 1, q - 3;
      Tdmatrix(j, j - 1) = -muy / 4
      Tdmatrix(j, j)     = 1;
      Tdmatrix(j, j + 1) = muy / 4    
      f(j) = Ftr(i - 1, j + 1) - (muy / 4) * (Ftr(i - 1, j + 2) - Ftr(i - 1, j))       
    end do

    !Решение на каждом шаге по времени методом прогонки
    !Нахождение прогоночных коэффициентов - прямой ход
    alpha(0) = -Tdmatrix(0, 1) / Tdmatrix(0, 0)
    beta(0) = f(0) / Tdmatrix(0, 0)
    do k = 1, q - 3
      alpha(k) = -Tdmatrix(k, k + 1) / (Tdmatrix(k, k) + alpha(k -1) * Tdmatrix(k, k - 1))
      beta(k) = (f(k) - beta(k - 1) * Tdmatrix(k, k - 1)) / (Tdmatrix(k, k) + alpha(k -1) * Tdmatrix(k, k - 1))
    end do  
    !Нахождение корней системы уравнений - обратный ход
    g = q - 2
    h = q - 3
    beta(g) = (f(g) - beta(h) * Tdmatrix(g, h)) / (Tdmatrix(g, g) + alpha(h) * Tdmatrix(g, h))
    RootOfEquation(g) = beta(g)
    do k = q - 3, 0, -1
      RootOfEquation(k) = alpha(k) * RootOfEquation(k + 1) + beta(k)
    end do

    !Заполнение значений функции в незаполненных узлах сетки
    do j = 1, q - 1
      Ftr(i, j) = RootOfEquation(j - 1)
    end do

  end do
        
  !Запись значений функций во всех узлах и координат по времени и пространству для построения графика
  do i = 0, s
    do j = 0, q
      t = i * dt
      r = j * dr
      write (1, 111) t/60., r, Ftr(i, j)
    end do
  end do
  111 format(f8.5, f10.1, f15.5)
  close (1)

end program CrankNicholsonScheme

