import java.util.*;
import java.io.*;
import java.lang.*;

/**
 * Crank-Nicholson Scheme. Linear one-dimensional advection equation.
 * @author ttnguyen 
 * Nguyen Tong Tam, group MB-457
 * Version 23.03.2010
 */

class CrankNicholsonScheme {

    public static void main(String[] args) throws IOException {

        final int s = 145;
        final int q = 101;
        int i;
        int j;
        int k;
        final double dr = 50;
        final double dt = 10;
        double r;
        double t;
        double m;
        double muy;
        final double C = 20;
        final double L = 1000;
        double[][] Ftr = new double[s][q];
        double[][] Tridiagonalmatrix = new double[q - 2][q - 2];
        double[] f = new double[q - 2];
        double[] RootOfEquation = new double[q - 2];
        double[] alpha = new double[q - 2];
        double[] beta = new double[q - 2];
        PrintStream outfile = new PrintStream(new FileOutputStream("CrankNicholsonScheme.txt"));

        //Волновое число и 
        m = 2 * Math.PI / L;
        muy = C * dt * 60 / (dr * 1000);

        //Задание начального условия
        for (j = 0; j < q; j++) {
          Ftr[0][j] = 50 * Math.cos(m * (j + 1) * dr) + 100;   
        }
        //Задание граничных условий
        for (i = 1; i < s; i++) {
          Ftr[i][0] = Ftr[0][0] + 0.0000005 * i * dt * 60;
          Ftr[i][q - 1] = Ftr[0][q - 1] + 0.0000005 * i * dt * 60; 
        }

        //Вычисление значений функции в узлах сетки на каждом шаге по времени
        
        for (i = 1; i < s; i++) {

          //Задание элементов трехдиагональной матрицы
          //Первая строка 
          Tridiagonalmatrix[0][0] = 1;
          Tridiagonalmatrix[0][1] = muy / 4;
          f[0] = Ftr[i - 1][1] - (muy / 4) * (-Ftr[i][0] + Ftr[i - 1][2] - Ftr[i - 1][0]);            
          //Подследняя строка
          Tridiagonalmatrix[q - 3][q - 4] = -muy / 4;
          Tridiagonalmatrix[q - 3][q - 3] = 1;
          f[q - 3] = Ftr[i - 1][q - 2] - (muy / 4) * (Ftr[i][q - 1] + Ftr[i - 1][q - 1] - Ftr[i - 1][q - 3]);          
          //j-ая строка
          for (j = 1; j < q - 3; j++) {
            Tridiagonalmatrix[j][j - 1] = -muy / 4;
            Tridiagonalmatrix[j][j]     = 1;
            Tridiagonalmatrix[j][j + 1] = muy / 4;    
            f[j] = Ftr[i - 1][j + 1] - (muy / 4) * (Ftr[i - 1][j + 2] - Ftr[i - 1][j]) ;          
          }

          //Решение на каждом шаге по времени методом прогонки
          //Нахождение прогоночных коэффициентов - прямой ход
          alpha[0] = -Tridiagonalmatrix[0][1] / Tridiagonalmatrix[0][0];
          beta[0] = f[0] / Tridiagonalmatrix[0][0];
          for (k = 1; k < q - 3; k++) {
            alpha[k] = -Tridiagonalmatrix[k][k + 1] / (Tridiagonalmatrix[k][k] + alpha[k -1] * Tridiagonalmatrix[k][k - 1]);
            beta[k] = (f[k] - beta[k - 1] * Tridiagonalmatrix[k][k - 1]) / (Tridiagonalmatrix[k][k] + alpha[k -1] * Tridiagonalmatrix[k][k - 1]);
          }
          //Нахождение корней системы уравнений - обратный ход
          beta[q - 3] = (f[q - 3] - beta[q - 4] * Tridiagonalmatrix[q - 3][q - 4]) / (Tridiagonalmatrix[q - 3][q - 3] + alpha[q - 4] * Tridiagonalmatrix[q - 3][q - 4]);
          RootOfEquation[q - 3] = beta[q - 3];
          for (k = q - 4; k >= 0; k--) {
            RootOfEquation[k] = alpha[k] * RootOfEquation[k + 1] + beta[k];
          }

          //Заполнение значений функции в незаполненных узлах сетки
          for (j = 1; j < q - 1; j++) {
            Ftr[i][j] = RootOfEquation[j - 1];
          }

        }

        //Запись значений функций во всех узлах и координат по времени и пространству для построения графика
        for (i = 0; i < s; i++) {
          for (j = 0; j < q; j++) {
            t = i * dt;
            r = j * dr;
            outfile.printf(Locale.US, "%.5f\t%.0f\t%.5f\n", t / 60.0, r, Ftr[i][j]);
          }
        }
        outfile.close();

    }

}
